--[[
LibGigaDebug-1.0 - Library providing simple debug logging for WoW addons
Copyright (C) 2008 GigaDrone

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
--]]

local MAJOR, MINOR = "LibGigaDebug-1.0", 100000 + tonumber(("$Revision: 1 $"):match("%d+"))
assert(LibStub, MAJOR.." requires LibStub")
local CBH = LibStub("CallbackHandler-1.0",false)
assert(CBH, MAJOR.." requires CallbackHandler-1.0")
local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end

lib.loggers = lib.loggers or {}
lib.debugging = lib.debugging or {}
lib.showFrame = lib.showFrame or {}
lib.mixins = lib.mixins or {}
lib.options = lib.options or {}
lib.callbacks = lib.callbacks or CBH:New(lib)	

function lib.callbacks:OnUsed(_, event)
	if event == "MessageLogged" then
		lib.quiet = true
	end
end

function lib.callbacks:OnUnused(_, event)
	if event == "MessageLogged" then
		lib.quiet = false
	end
end

local loggers = lib.loggers
local debugging = lib.debugging
local showFrame = lib.showFrame
local mixins = lib.mixins
local options = lib.options
local msgs = ''

function mixins:ToggleDebugLog(value, sFrame)
	value = not not value
	if sFrame ~= showFrame[self] then		
		showFrame[self] = sFrame
		if value then
			if showFrame[self] then
				lib:CreateDEBUGFrame()
			end
		end
		if not showFrame[self] and lib.DEBUGFrame then
			lib.DEBUGFrame:Hide()
		end
	end
	if value ~= debugging[self] then
		debugging[self] = value
		lib.callbacks:Fire(value and 'DebugLogEnabled' or 'DebugLogDisabled', self, value)
	end
end

function mixins:IsDebugLogEnabled()
	return debugging[self]
end

do
	-- Based on AceDebug-2.0
	local tmp = {}
	local function LogMessage(self, a1, ...)
		if lib.globalToggle == false or (lib.globalToggle == nil and not debugging[self]) then return end
		local message, now, n = "", GetTime(), select('#', ...)
		local tempValue, color = '','B9B9FF'
		a1 = tostring(a1)
		if a1:find("%%") and n > 0 then
			for i=1,n do 
				tempValue = select(i, ...)
				if type(tempValue) == "boolean" then
					if tempValue then
						color = "55ff55"
						tempValue = "true"
					else
						color = "ff5555"
						tempValue = "false"
					end
				else
					if tempValue then
						color = "B9B9FF"
						tempValue = tostring(tempValue)
						if string.lower(tempValue) == "false" or string.lower(tempValue) == "nil" or string.lower(tempValue) == "no" then
							color = "ff5555"
						elseif string.lower(tempValue) == "true" or string.lower(tempValue) == "yes" then
							color = "55ff55"
						end
					else
						color = "ff5555"
						tempValue = "nil"
					end
				end
				tmp[i] = "|cff"..color..tempValue.."|r" 
			end
			message = a1:format(unpack(tmp))
		else
			for i=1,n do 
				tmp[i] = "|cff55ff55"..tostring(select(i, ...)).."|r" 
			end
			message = a1 .. " " .. table.concat(tmp, ", ")
			wipe(tmp)
		end
		
		if lib.quiet then
			lib.callbacks:Fire("MessageLogged", self, now, message)
		else
			local milisec = tostring(floor((now % 1) * 1000))
			local solomsgs = ''
			milisec = string.len(milisec)<2 and ('00'..milisec) or (string.len(milisec)<3 and ('0'..milisec) or milisec)
			
			solomsgs = ("|cff555555[%s'%s]|r: %s"):format(date("%H:%M:%S"), milisec, message)
			msgs = solomsgs.."\n"..msgs
			
			if showFrame[self] then
				lib.DEBUGFrame.Title:SetText("DEBUG: "..tostring(self))
				lib.DEBUGFrame.EditBox:SetText(msgs)
				lib.DEBUGFrame:Show()
			else
				DEFAULT_CHAT_FRAME:AddMessage(solomsgs)
			end		
		end
	end
	
	function mixins:Debug(...)
		local success, errorMsg = pcall(LogMessage, self, ...)
		if not success then
			geterrorhandler()(errorMsg)
		end
	end
end

function lib:SetGlobalToggle(value)
	if value ~= nil then
		value = not not value
	end
	if value ~= lib.globalToggle then
		lib.globalToggle = value
		lib.callbacks:Fire(value and 'GlobalDebugEnabled' or 'GlobalDebugDisabled', value)
	end
end

function lib:GetGlobalToggle()
	return lib.globalToggle
end

function lib:GetAce3OptionTable(logger, order)
	if not loggers[logger] then return end
	if not options[logger] then
		options[logger] = {
			name = 'Enable debug logs',
			type = 'toggle',
			get = function(info) return logger:IsDebugLogEnabled() end,
			set = function(info, value) return logger:ToggleDebugLog(value) end,
			disabled = function() return lib.globalToggle ~= nil end,
			order = order or 100,
		}
	end
	return options[logger]
end

do
	local function iter(onlyNamed, logger)
		while true do
			local name
			logger, name = next(loggers, logger)
			if not logger then
				return
			elseif not onlyNamed or name ~= true then
				return logger, name
			end
		end
	end
	function lib:IterateLoggers(onlyNamed)
		return iter, onlyNamed
	end
end

function lib:Embed(logger)
	assert(type(logger) == 'table')
	for name,func in pairs(mixins) do
		logger[name] = func
	end
	if not loggers[logger] then
		local name = rawget(logger, 'name')
		loggers[logger] = name or true
		self.callbacks:Fire('NewLogger', logger, name)
		self.callbacks:Fire('NewBroker', logger, name) -- Compat
	end
end

function lib:GetLoggerName(logger)
	local name = loggers[logger]
	return name ~= true and name or nil
end

function lib:CreateDEBUGFrame()
    
	if self.DEBUGFrame then return end
	
	local sfont = "Interface\\AddOns\\GigaItemAuction\\textures\\FRIZQT__.TTF"
	
	self.DEBUGFrame = CreateFrame("Frame", "DEBUGFrame", UIParent)
    local f = self.DEBUGFrame
	
	f.isResizing = false
	
	f:SetFrameStrata("HIGH")
	f:SetResizable(true)
    f:SetMovable(true)
    f:SetWidth(600)
    f:SetHeight(300)
    f:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background",
        edgeFile =  "Interface/Tooltips/UI-Tooltip-Border",
        edgeSize= 16, insets = { left = 0, right = 0, top = 0, bottom = 0 }}) 
    f:SetBackdropColor(			unpack({	0,		0,		0,		1	}))
    f:SetBackdropBorderColor(	unpack({	0.4,	0,		0,		1	}))
    f:ClearAllPoints()
	f:RegisterForDrag("LeftButton")
    f:SetScript("OnMouseDown", function() lib.DEBUGFrame:StartMoving() end)
    f:SetScript("OnMouseUp", function() lib.DEBUGFrame:StopMovingOrSizing() end)
    f:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 40, -40)
	
--~ 	Titel
	f.Title = f:CreateFontString("GIGA_Title", "HIGH")
    f.Title:SetFont(sfont,14)
    f.Title:SetText("DEBUG")
    f.Title:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -5)
    f.Title:SetTextColor(unpack({	1,		1,		0,		1	}))
--~     closeButton
    f.closeButton = CreateFrame("Button", nil, f, "UIPanelCloseButton")
    f.closeButton:SetPoint("TOPRIGHT", f, "TOPRIGHT", 3, 3)
    f.closeButton:SetWidth(30)
    f.closeButton:SetHeight(30)    
    f.closeButton:SetScript("OnClick", function() f:Hide() end)
	
    f.adminButton = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
    f.adminButton:SetPoint("RIGHT", f.closeButton, "LEFT", 5, 0)
    f.adminButton:SetWidth(48)
    f.adminButton:SetHeight(16)  
    f.adminButton:SetText("Clear") 
	f.adminButton:RegisterForClicks("AnyUp")
    f.adminButton:SetScript("OnClick", function() msgs = ''; f.EditBox:SetText(msgs); end)
	
	
    f.scroll = CreateFrame("ScrollFrame", "GIAScrollFrame", f, "UIPanelScrollFrameTemplate")
	f.scroll:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -25)
	f.scroll:SetPoint("RIGHT", f, "RIGHT", -25, 0)
	f.scroll:SetPoint("BOTTOM", f, 0, 10)

	f.EditBox = CreateFrame("EditBox", "GIAEditBox", f.scroll)
	f.EditBox:SetFont(sfont,14)
	f.EditBox:SetMultiLine(true)
	f.EditBox:SetWidth(f:GetWidth()-40)
	f.EditBox:SetScript("OnEscapePressed", function() this:ClearFocus() end)
	f.EditBox:SetScript("OnTextChanged", function() f.scroll:UpdateScrollChildRect() end)
    f.EditBox:SetScript("OnEditFocusGained", function() this:ClearFocus() end)
	f.EditBox:SetText("")
	f.EditBox:SetAutoFocus(false)
	f.EditBox:ClearFocus()
	f.scroll:SetScrollChild(f.EditBox)
	
	
	f.DragRight = CreateFrame("Button", nil, f)
	f.DragRight:SetFrameLevel( f:GetFrameLevel() + 10)
	f.DragRight:SetNormalTexture("Interface\\AddOns\\GigaItemAuction\\textures\\ResizeGripRight")
	f.DragRight:SetHighlightTexture("Interface\\AddOns\\GigaItemAuction\\textures\\ResizeGripRight")
	f.DragRight:SetWidth(16)
	f.DragRight:SetHeight(16)
	f.DragRight:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 5, -5)
	f.DragRight:EnableMouse(true)
	f.DragRight:SetScript("OnMouseDown", 
        function() 
            f.isResizing = true; 
			f:StartSizing("BOTTOMRIGHT") 
        end )
	f.DragRight:SetScript("OnMouseUp", 
        function() 
            if f.isResizing == true then 
                f:StopMovingOrSizing()
				f:SetPoint("LEFT", a, "RIGHT")
				f:SetWidth(f:GetWidth())
				f.EditBox:SetWidth(f:GetWidth()-40)
				f:SetHeight(f:GetHeight())
                f.isResizing = false;
            end 
        end )
	f:Hide()
end

lib.GetBrokerName = lib.GetLoggerName -- Compat
lib.IterateBrokers = lib.IterateLoggers -- Compat

for logger in pairs(loggers) do
	lib:Embed(logger)
end

